#include "operations.h"

#include <RobotAPI/libraries/armem/client/query/Builder.h>
#include <RobotAPI/libraries/armem/server/ltm/disk/MemoryManager.h>
#include <RobotAPI/libraries/armem/server/query_proc/ltm.h>
#include <RobotAPI/libraries/armem/server/wm/memory_definitions.h>


namespace armarx::armem::server::ltm
{

    armem::wm::Memory
    disk::load(const std::filesystem::path& directory)
    {
        const std::string key = directory.filename();
        armem::server::ltm::disk::MemoryManager manager;
        manager.setName(key);
        manager.setBasePath(directory);
        manager.reload();

        armem::client::QueryBuilder builder;
        builder.all();
        armem::client::QueryInput queryInput = builder.buildQueryInput();
        queryInput.addQueryTargetToAll(armem::query::data::QueryTarget::LTM);

        armem::server::query_proc::ltm::MemoryQueryProcessor processor;
        armem::wm::Memory memory =
            processor.process(queryInput.toIce(), manager.getCacheAndLutNotConverted());
        manager.convert(memory);

        return memory;
    }


    namespace detail
    {
        template <class MemoryT>
        void
        store(const std::filesystem::path& directory, const MemoryT& memory)
        {
            const std::string& name = memory.name();
            if (std::filesystem::is_regular_file(directory / name))
            {
                std::stringstream ss;
                ss << "Could not export memory '" << name << "' to " << directory << ": "
                   << "Cannot overwrite existing file.\n";
                throw error::IOError(directory, ss.str());
            }
            else
            {
                std::filesystem::create_directories(directory / name);

                armem::server::ltm::disk::MemoryManager manager;
                manager.setName(name);
                manager.setBasePath(directory / name);
                manager.reload();
                manager.append(memory);
            }
        }
    } // namespace detail


    void
    disk::store(const std::filesystem::path& directory, const armem::wm::Memory& memory)
    {
        detail::store(directory, memory);
    }


    void
    disk::store(const std::filesystem::path& directory, const armem::server::wm::Memory& memory)
    {
        detail::store(directory, memory);
    }

} // namespace armarx::armem::server::ltm
